import unittest

class TestTextToHTMLparser(unittest.TestCase):
    def test_convert_linebreaks(self):
        # text_to_htm_lparser = TextToHTMLparser()
        # self.assertEqual(expected, text_to_htm_lparser.convert_linebreaks(text))
        assert True # TODO: implement your test here

    def test_convert_urls(self):
        # text_to_htm_lparser = TextToHTMLparser()
        # self.assertEqual(expected, text_to_htm_lparser.convert_urls(text))
        assert True # TODO: implement your test here

    def test_do_sub(self):
        # text_to_htm_lparser = TextToHTMLparser()
        # self.assertEqual(expected, text_to_htm_lparser.do_sub(m))
        assert True # TODO: implement your test here

    def test_parse(self):
        # text_to_htm_lparser = TextToHTMLparser()
        # self.assertEqual(expected, text_to_htm_lparser.parse(text, strip_ansi))
        assert True # TODO: implement your test here

    def test_re_bold(self):
        # text_to_htm_lparser = TextToHTMLparser()
        # self.assertEqual(expected, text_to_htm_lparser.re_bold(text))
        assert True # TODO: implement your test here

    def test_re_color(self):
        # text_to_htm_lparser = TextToHTMLparser()
        # self.assertEqual(expected, text_to_htm_lparser.re_color(text))
        assert True # TODO: implement your test here

    def test_re_underline(self):
        # text_to_htm_lparser = TextToHTMLparser()
        # self.assertEqual(expected, text_to_htm_lparser.re_underline(text))
        assert True # TODO: implement your test here

    def test_remove_backspaces(self):
        # text_to_htm_lparser = TextToHTMLparser()
        # self.assertEqual(expected, text_to_htm_lparser.remove_backspaces(text))
        assert True # TODO: implement your test here

    def test_remove_bells(self):
        # text_to_htm_lparser = TextToHTMLparser()
        # self.assertEqual(expected, text_to_htm_lparser.remove_bells(text))
        assert True # TODO: implement your test here

class TestParseHtml(unittest.TestCase):
    def test_parse_html(self):
        # self.assertEqual(expected, parse_html(string, strip_ansi, parser))
        assert True # TODO: implement your test here

if __name__ == '__main__':
    unittest.main()
